#include "LoRaWAN.h"

#if !defined(RADIOLIB_EXCLUDE_LORAWAN)

uint8_t getDownlinkDataRate(uint8_t uplink, uint8_t offset, uint8_t base, uint8_t lim) {
  int8_t dr = uplink - offset + base;
  if(dr < lim) {
    dr = lim;
  }
  return(dr);
}

const LoRaWANBand_t EU868 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 230, 230,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 16,
  .powerNumSteps = 7,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 3,
      .freqStart = 868.1,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_250_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 869.858,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t US915 = {
  .downlinkDataRateBase = 10,
  .downlinkDataRateMin = 8,
  .payloadLenMax = { 
     19,  61, 133, 250, 250,   0,   0,   0,
     41, 117, 230, 230, 230, 230,   0,   0 },
  .powerMax = 30,
  .powerNumSteps = 10,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_MASK,
  .numChannelSpans = 3,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_UPLINK,
      .joinRequestDataRate = 0,
      .numChannels = 64,
      .freqStart = 902.3,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    }, {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_UPLINK,
      .joinRequestDataRate = 4,
      .numChannels = 8,
      .freqStart = 903,
      .freqStep = 1.6,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    }, {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 8,
      .freqStart = 923.3,
      .freqStep = 0.6,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    },
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 923.3,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t CN780 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 250, 230,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 12,
  .powerNumSteps = 5,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 6,
      .freqStart = 779.5,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_250_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 786,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t EU433 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 230, 230,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 12,
  .powerNumSteps = 5,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 3,
      .freqStart = 433.175,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_250_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 434.665,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t AU915 = {
  .downlinkDataRateBase = 8,
  .downlinkDataRateMin = 8,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 230,   0,
     41, 117, 230, 230, 230, 230,   0,   0 },
  .powerMax = 30,
  .powerNumSteps = 10,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_MASK,
  .numChannelSpans = 3,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_UPLINK,
      .joinRequestDataRate = 0,
      .numChannels = 64,
      .freqStart = 915.2,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    }, {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_UPLINK,
      .joinRequestDataRate = 6,
      .numChannels = 8,
      .freqStart = 915.9,
      .freqStep = 1.6,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    }, {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 8,
      .freqStart = 923.3,
      .freqStep = 0.6,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    },
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 923.3,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_500_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t CN500 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230,   0,   0,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 19,
  .powerNumSteps = 7,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_MASK,
  .numChannelSpans = 2,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_UPLINK,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 96,
      .freqStart = 470.3,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    }, {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 48,
      .freqStart = 500.3,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 505.3,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t AS923 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 230, 230,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 16,
  .powerNumSteps = 7,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = 2,
      .numChannels = 2,
      .freqStart = 923.2,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_250_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 923.2,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t KR920 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230,   0,   0,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 14,
  .powerNumSteps = 7,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 3,
      .freqStart = 922.1,
      .freqStep = 0.2,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_250_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 921.9,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

const LoRaWANBand_t IN865 = {
  .downlinkDataRateBase = 0,
  .downlinkDataRateMin = 0,
  .payloadLenMax = { 
     59,  59,  59, 123, 230, 230, 230, 230,
      0,   0,   0,   0,   0,   0,   0,   0 },
  .powerMax = 30,
  .powerNumSteps = 10,
  .cfListType = RADIOLIB_LORAWAN_CFLIST_TYPE_FREQUENCIES,
  .numChannelSpans = 1,
  .defaultChannels = {
    {
      .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_BOTH,
      .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      .numChannels = 3,
      .freqStart = 865.0625,
      .freqStep = 0.36,
      .dataRates = {
        RADIOLIB_LORAWAN_DATA_RATE_SF_12 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_11 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_9 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_8 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_SF_7 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_FSK_50_K,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
        RADIOLIB_LORAWAN_DATA_RATE_UNUSED
      }
    },
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
    RADIOLIB_LORAWAN_CHANNEL_SPAN_NONE,
  },
  .backupChannel = {
    .direction = RADIOLIB_LORAWAN_CHANNEL_DIR_DOWNLINK,
    .joinRequestDataRate = RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    .numChannels = 1,
    .freqStart = 866.55,
    .freqStep = 0,
    .dataRates = {
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_SF_10 | RADIOLIB_LORAWAN_DATA_RATE_BW_125_KHZ,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
      RADIOLIB_LORAWAN_DATA_RATE_UNUSED,
    }
  }
};

#endif
